package com.astro.bibliotheca.impl;

import com.astro.bibliotheca.api.BiblioPlugin;
import com.astro.bibliotheca.api.BibliothecaAPI;
import com.astro.bibliotheca.api.BibliothecaPlugin;
import com.astro.bibliotheca.api.worldenough.SpawnLogic;
import com.astro.bibliotheca.worldenough.impl.ImplWEApi;
import com.astro.bibliotheca.worldenough.json.JSONReader;
import com.astro.bibliotheca.worldenough.json.JSONWriter;
import com.astro.bibliotheca.worldenough.world.WorldEnoughGenerator;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.terraingen.OreGenEvent;
import net.minecraftforge.fml.common.ProgressManager;
import net.minecraftforge.fml.common.discovery.ASMDataTable;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.GameRegistry;

import java.util.Set;

public class ImplBiblioAPI implements BibliothecaAPI {
    private ImplWEApi worldEnoughAPI = new ImplWEApi();

    @Override
    public ImplWEApi getWorldEnoughAPI() {
        return worldEnoughAPI;
    }

    public void init(FMLPreInitializationEvent event) {
        MinecraftForge.EVENT_BUS.register(this);
        MinecraftForge.ORE_GEN_BUS.register(this);

        Set<ASMDataTable.ASMData> datas = event.getAsmData().getAll(BiblioPlugin.class.getCanonicalName());
        ProgressManager.ProgressBar modBar = ProgressManager.push("Loading Plugins", datas.size(), true);
        for (final ASMDataTable.ASMData asmData : datas) {
            final String value = asmData.getAnnotationInfo().get("value").toString();
            modBar.step(value);
            try {
                worldEnoughAPI.registerSpawnLogic(value, Class.forName(asmData.getClassName()).asSubclass(BibliothecaPlugin.class).newInstance().registerWorldGen(worldEnoughAPI));
            } catch (ClassNotFoundException | IllegalAccessException | InstantiationException e) {

            }
        }
        JSONWriter.write();

        JSONReader.read();

        for (SpawnLogic logic : worldEnoughAPI.getAllSpawnLogic().values())
            GameRegistry.registerWorldGenerator(new WorldEnoughGenerator(logic), 0);
        ProgressManager.pop(modBar);
    }

    @SubscribeEvent
    public void onGenerateMinable(OreGenEvent.GenerateMinable event) {
        event.setResult(Event.Result.DENY);
    }
}
