package com.astro.bibliotheca.pulsar;

import com.astro.bibliotheca.pulsar.content.*;
import net.minecraft.block.Block;
import net.minecraft.client.Minecraft;
import net.minecraft.item.Item;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.client.event.ModelRegistryEvent;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.Locale;

@SuppressWarnings({"unused", "WeakerAccess"})
public abstract class BasePulse {

    protected final ForgeRegistryList<Item> items = new ForgeRegistryList<>();
    protected final ForgeRegistryList<Block> blocks = new ForgeRegistryList<>();

    protected static boolean isModLoaded(String modid) {
        return Loader.isModLoaded(modid);
    }

    protected static void registerTE(Class<? extends TileEntity> teClazz, String name) {
        if (!name.equals(name.toLowerCase(Locale.US)))
            throw new IllegalArgumentException(String.format("Unlocalized names need to be all lowercase! TE: %s", name));
        GameRegistry.registerTileEntity(teClazz, name);
    }

    @SideOnly(Side.CLIENT)
    protected void colourRegistrationInit() {
        items.stream()
                .filter(IColorProviderItem.class::isInstance)
                .map(i -> (Item & IColorProviderItem) i)
                .forEach(i -> Minecraft.func_71410_x().getItemColors().func_186730_a(i.getColor(), i));
        blocks.stream()
                .filter(IColorProvider.class::isInstance)
                .map(b -> (Block & IColorProvider) b)
                .forEach(b -> {
                    Minecraft.func_71410_x().getItemColors().func_186730_a(b.getItemColor(), Item.func_150898_a(b));
                    Minecraft.func_71410_x().func_184125_al().func_186722_a(b.getColor(), b);
                });
    }

    protected void addItem(Item item) {
        items.add(item);
    }

    protected void addItems(Item... items) {
        for (Item item : items)
            addItem(item);
    }

    protected void addBlock(Block block) {
        blocks.add(block);
        if (block instanceof IItemBlockProvider)
            addItem(((IItemBlockProvider) block).createItemBlock());
    }

    protected void addBlocks(Block... blocks) {
        for (Block block : blocks)
            addBlock(block);
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public void registerModels(ModelRegistryEvent event) {
        if (!blocks.isEmpty())
            blocks.stream()
                    .filter(IModelRegister.class::isInstance)
                    .map(IModelRegister.class::cast)
                    .forEach(b -> b.initModels(event));
        if (!items.isEmpty())
            items.stream()
                    .filter(IModelRegister.class::isInstance)
                    .map(IModelRegister.class::cast)
                    .forEach(i -> i.initModels(event));
    }

    @SubscribeEvent
    public void registerItems(RegistryEvent.Register<Item> event) {
        items.register(event.getRegistry());
    }

    @SubscribeEvent
    public void registerBlocks(RegistryEvent.Register<Block> event) {
        blocks.register(event.getRegistry());
    }
}
