package com.astro.bibliotheca.pulsar.config;

import com.astro.bibliotheca.pulsar.pulse.PulseMeta;
import mcp.MethodsReturnNonnullByDefault;
import net.minecraftforge.common.config.ConfigCategory;
import net.minecraftforge.common.config.Configuration;
import net.minecraftforge.common.config.Property;
import net.minecraftforge.fml.common.Loader;

import javax.annotation.ParametersAreNonnullByDefault;
import java.io.File;
import java.util.Locale;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class ForgeConfiguration implements IConfiguration {
    private final String description;
    private Configuration config;

    public ForgeConfiguration(String confName, String description) {
        this.description = description.toLowerCase(Locale.US);
        if (!confName.endsWith(".cfg"))
            confName += ".cfg";
        config = new Configuration(new File(Loader.instance().getConfigDir().toString() + File.separator + confName), "1");
    }

    @Override
    public void load() {
        config.load();
    }

    @Override
    public boolean isModuleEnabled(PulseMeta meta) {
        Property prop = config.get(this.description, meta.getId(), meta.isDefaultEnabled(), meta.getDescription());
        prop.setRequiresMcRestart(true);
        return prop.getBoolean(meta.isEnabled());
    }

    @Override
    public void flush() {
        if (config.hasChanged())
            config.save();
    }

    public Configuration getConfig() {
        return config;
    }

    public ConfigCategory getCategory() {
        return config.getCategory(this.description);
    }
}
