package com.astro.bibliotheca.pulsar.flightpath.dispatch;

import com.astro.bibliotheca.pulsar.flightpath.IDispatcher;
import com.astro.bibliotheca.pulsar.flightpath.IExceptionHandler;
import com.astro.bibliotheca.pulsar.flightpath.lib.Pair;

import javax.annotation.Nonnull;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;

/**
 * Basic reflection-based dispatcher.
 */
public class JavaDispatcher extends IDispatcher {

    private Map<Class, List<Pair<Object, Set<Method>>>> registeredListeners = new HashMap<>();

    public JavaDispatcher(IExceptionHandler exceptionHandler) {
        super(exceptionHandler);
    }

    @Override
    public void addSubscriber(@Nonnull Object target, @Nonnull Map<Class, Set<Method>> locatedMethods) {
        synchronized (lock) {
            for (Map.Entry<Class, Set<Method>> entry : locatedMethods.entrySet()) {
                List<Pair<Object, Set<Method>>> current = registeredListeners.get(entry.getKey());
                if (current == null) current = new ArrayList<>();
                current.add(new Pair<>(target, entry.getValue()));
                registeredListeners.put(entry.getKey(), current);
            }
        }
    }

    @Override
    public void dispatch(@Nonnull Object evt) {
        synchronized (lock) {
            List<Pair<Object, Set<Method>>> listeners = registeredListeners.get(evt.getClass());
            if (listeners == null) return;
            for (Pair<Object, Set<Method>> pair : listeners) {
                try {
                    Object target = pair.a;
                    Set<Method> methods = pair.b;
                    for (Method m : methods) {
                        boolean access = m.isAccessible();
                        m.setAccessible(true);
                        m.invoke(target, evt);
                        m.setAccessible(access);
                    }
                } catch (IllegalAccessException | InvocationTargetException ex) {
                    throw new RuntimeException(ex);
                }
            }
        }
    }

}
