package com.astro.bibliotheca.util;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.ItemModelMesher;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.client.renderer.block.statemap.DefaultStateMapper;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.client.model.ModelLoader;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.lwjgl.opengl.GL11;

@SideOnly(Side.CLIENT)
public class ClientUtil {

    private static final float fluidOffset = 0.005F;

    public static void registerModel(Item item, String location) {
        ItemModelMesher mesher = Minecraft.func_71410_x().func_175599_af().func_175037_a();
        mesher.func_178086_a(item, 0, new ModelResourceLocation(new ResourceLocation(location), "inventory"));
    }

    public static void registerModel(Block block, String location) {
        ItemModelMesher mesher = Minecraft.func_71410_x().func_175599_af().func_175037_a();
        mesher.func_178086_a(Item.func_150898_a(block), 0, new ModelResourceLocation(new ResourceLocation(location), "inventory"));
    }

    public static void registerWithMapper(Block block) {
        if (Platform.isClient() && block != null) {
            final String resourcePath = block.getRegistryName().toString();

            ModelLoader.setCustomStateMapper(block, new DefaultStateMapper() {
                @Override
                protected ModelResourceLocation func_178132_a(IBlockState state) {
                    return new ModelResourceLocation(resourcePath, func_178131_a(state.func_177228_b()));
                }
            });

            NonNullList<ItemStack> subBlocks = NonNullList.func_191196_a();
            block.func_149666_a(null, subBlocks);

            for (ItemStack stack : subBlocks) {
                IBlockState state = block.func_176203_a(stack.func_77960_j());
                ModelLoader.setCustomModelResourceLocation(Item.func_150898_a(block), stack.func_77960_j(), new ModelResourceLocation(resourcePath, Platform.getPropertyString(state.func_177228_b())));
            }
        }
    }

    public static void setCustomModelLocation(Item item) {
        ModelResourceLocation location = new ModelResourceLocation(item.getRegistryName(), "inventory");
        ModelLoader.setCustomModelResourceLocation(item, 0, location);
    }

    public static void setCustomModelLocation(Block block) {
        ModelResourceLocation location = new ModelResourceLocation(block.getRegistryName(), "inventory");
        ModelLoader.setCustomModelResourceLocation(Item.func_150898_a(block), 0, location);
    }

    public static void renderTexturedQuad(TextureAtlasSprite texture, EnumFacing face, BlockPos pos, double x1, double y1, double z1, double x2, double y2, double z2, int brightness, int color) {
        BufferBuilder renderer = Tessellator.func_178181_a().func_178180_c();
        renderer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
        Minecraft.func_71410_x().field_71446_o.func_110577_a(TextureMap.field_110575_b);
        putTexturedQuad(renderer, texture, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, face, color, brightness, false);
        Tessellator.func_178181_a().func_78381_a();
    }

    public static void renderTexturedCuboid(TextureAtlasSprite[] textures, BlockPos pos, double x1, double y1, double z1, double x2, double y2, double z2, int brightness, int color) {
        BufferBuilder renderer = Tessellator.func_178181_a().func_178180_c();
        renderer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
        Minecraft.func_71410_x().field_71446_o.func_110577_a(TextureMap.field_110575_b);
        putTexturedQuad(renderer, textures[0], x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.DOWN, color, brightness, false);
        putTexturedQuad(renderer, textures[1], x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.NORTH, color, brightness, false);
        putTexturedQuad(renderer, textures[2], x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.EAST, color, brightness, false);
        putTexturedQuad(renderer, textures[3], x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.SOUTH, color, brightness, false);
        putTexturedQuad(renderer, textures[4], x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.WEST, color, brightness, false);
        putTexturedQuad(renderer, textures[5], x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.UP, color, brightness, false);
        Tessellator.func_178181_a().func_78381_a();
    }

    public static void renderTexturedCuboid(TextureAtlasSprite texture, BlockPos pos, double x1, double y1, double z1, double x2, double y2, double z2, int brightness, int color) {
        BufferBuilder renderer = Tessellator.func_178181_a().func_178180_c();
        renderer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
        Minecraft.func_71410_x().field_71446_o.func_110577_a(TextureMap.field_110575_b);
        putTexturedQuad(renderer, texture, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.DOWN, color, brightness, false);
        putTexturedQuad(renderer, texture, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.NORTH, color, brightness, false);
        putTexturedQuad(renderer, texture, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.EAST, color, brightness, false);
        putTexturedQuad(renderer, texture, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.SOUTH, color, brightness, false);
        putTexturedQuad(renderer, texture, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.WEST, color, brightness, false);
        putTexturedQuad(renderer, texture, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.UP, color, brightness, false);
        Tessellator.func_178181_a().func_78381_a();
    }


    public static void drawCube(final double minX, final double minY, final double minZ, final double maxX, final double maxY, final double maxZ, final BufferBuilder buffer) {
        drawPlaneNegX(minX, minY, maxY, minZ, maxZ, buffer);
        drawPlanePosX(maxX, minY, maxY, minZ, maxZ, buffer);
        drawPlaneNegY(minY, minX, maxX, minZ, maxZ, buffer);
        drawPlanePosY(maxY, minX, maxX, minZ, maxZ, buffer);
        drawPlaneNegZ(minZ, minX, maxX, minY, maxY, buffer);
        drawPlanePosZ(maxZ, minX, maxX, minY, maxY, buffer);
    }


    public static void drawCube(final double minX, final double minY, final double minZ, final double maxX, final double maxY, final double maxZ, final float r, final float g, final float b, final float a, final BufferBuilder buffer) {
        drawPlaneNegX(minX, minY, maxY, minZ, maxZ, r, g, b, a * 0.9f, buffer);
        drawPlanePosX(maxX, minY, maxY, minZ, maxZ, r, g, b, a * 0.9f, buffer);
        drawPlaneNegY(minY, minX, maxX, minZ, maxZ, r, g, b, a * 0.8f, buffer);
        drawPlanePosY(maxY, minX, maxX, minZ, maxZ, r, g, b, a * 1.1f, buffer);
        drawPlaneNegZ(minZ, minX, maxX, minY, maxY, r, g, b, a, buffer);
        drawPlanePosZ(maxZ, minX, maxX, minY, maxY, r, g, b, a, buffer);
    }


    public static void drawPlaneNegX(final double x, final double minY, final double maxY, final double minZ, final double maxZ, final BufferBuilder buffer) {
        buffer.func_181662_b(x, minY, minZ).func_181675_d();
        buffer.func_181662_b(x, minY, maxZ).func_181675_d();
        buffer.func_181662_b(x, maxY, maxZ).func_181675_d();
        buffer.func_181662_b(x, maxY, minZ).func_181675_d();
    }


    public static void drawPlaneNegX(final double x, final double minY, final double maxY, final double minZ, final double maxZ, final float r, final float g, final float b, final float a, final BufferBuilder buffer) {
        buffer.func_181662_b(x, minY, minZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(x, minY, maxZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(x, maxY, maxZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(x, maxY, minZ).func_181666_a(r, g, b, a).func_181675_d();
    }


    public static void drawPlanePosX(final double x, final double minY, final double maxY, final double minZ, final double maxZ, final BufferBuilder buffer) {
        buffer.func_181662_b(x, minY, minZ).func_181675_d();
        buffer.func_181662_b(x, maxY, minZ).func_181675_d();
        buffer.func_181662_b(x, maxY, maxZ).func_181675_d();
        buffer.func_181662_b(x, minY, maxZ).func_181675_d();
    }


    public static void drawPlanePosX(final double x, final double minY, final double maxY, final double minZ, final double maxZ, final float r, final float g, final float b, final float a, final BufferBuilder buffer) {
        buffer.func_181662_b(x, minY, minZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(x, maxY, minZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(x, maxY, maxZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(x, minY, maxZ).func_181666_a(r, g, b, a).func_181675_d();
    }


    public static void drawPlaneNegY(final double y, final double minX, final double maxX, final double minZ, final double maxZ, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, y, minZ).func_181675_d();
        buffer.func_181662_b(maxX, y, minZ).func_181675_d();
        buffer.func_181662_b(maxX, y, maxZ).func_181675_d();
        buffer.func_181662_b(minX, y, maxZ).func_181675_d();
    }


    public static void drawPlaneNegY(final double y, final double minX, final double maxX, final double minZ, final double maxZ, final float r, final float g, final float b, final float a, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, y, minZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, y, minZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, y, maxZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(minX, y, maxZ).func_181666_a(r, g, b, a).func_181675_d();
    }


    public static void drawPlanePosY(final double y, final double minX, final double maxX, final double minZ, final double maxZ, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, y, minZ).func_181675_d();
        buffer.func_181662_b(minX, y, maxZ).func_181675_d();
        buffer.func_181662_b(maxX, y, maxZ).func_181675_d();
        buffer.func_181662_b(maxX, y, minZ).func_181675_d();
    }


    public static void drawPlanePosY(final double y, final double minX, final double maxX, final double minZ, final double maxZ, final float r, final float g, final float b, final float a, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, y, minZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(minX, y, maxZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, y, maxZ).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, y, minZ).func_181666_a(r, g, b, a).func_181675_d();
    }


    public static void drawPlaneNegZ(final double z, final double minX, final double maxX, final double minY, final double maxY, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, minY, z).func_181675_d();
        buffer.func_181662_b(minX, maxY, z).func_181675_d();
        buffer.func_181662_b(maxX, maxY, z).func_181675_d();
        buffer.func_181662_b(maxX, minY, z).func_181675_d();
    }


    public static void drawPlaneNegZ(final double z, final double minX, final double maxX, final double minY, final double maxY, final float r, final float g, final float b, final float a, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, minY, z).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(minX, maxY, z).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, maxY, z).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, minY, z).func_181666_a(r, g, b, a).func_181675_d();
    }


    public static void drawPlanePosZ(final double z, final double minX, final double maxX, final double minY, final double maxY, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, minY, z).func_181675_d();
        buffer.func_181662_b(maxX, minY, z).func_181675_d();
        buffer.func_181662_b(maxX, maxY, z).func_181675_d();
        buffer.func_181662_b(minX, maxY, z).func_181675_d();
    }


    public static void drawPlanePosZ(final double z, final double minX, final double maxX, final double minY, final double maxY, final float r, final float g, final float b, final float a, final BufferBuilder buffer) {
        buffer.func_181662_b(minX, minY, z).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, minY, z).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(maxX, maxY, z).func_181666_a(r, g, b, a).func_181675_d();
        buffer.func_181662_b(minX, maxY, z).func_181666_a(r, g, b, a).func_181675_d();
    }

    public static void renderFluidCuboid(FluidStack fluid, BlockPos pos, double x, double y, double z, double w, double h, double d) {
        double wd = (1d - w) / 2d;
        double hd = (1d - h) / 2d;
        double dd = (1d - d) / 2d;
        renderFluidCuboid(fluid, pos, x, y, z, wd, hd, dd, 1d - wd, 1d - hd, 1d - dd);
    }

    public static void renderFluidCuboid(FluidStack fluid, BlockPos pos, double x, double y, double z, double x1, double y1, double z1, double x2, double y2, double z2) {
        int color = fluid.getFluid().getColor(fluid);
        renderFluidCuboid(fluid, pos, x, y, z, x1, y1, z1, x2, y2, z2, color);
    }

    public static void renderFluidCuboid(FluidStack fluid, BlockPos pos, double x, double y, double z, double x1, double y1, double z1, double x2, double y2, double z2, int color) {
        BufferBuilder renderer = Tessellator.func_178181_a().func_178180_c();
        renderer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
        Minecraft.func_71410_x().field_71446_o.func_110577_a(TextureMap.field_110575_b);
        int brightness = Minecraft.func_71410_x().field_71441_e.func_175626_b(pos, fluid.getFluid().getLuminosity());
        TextureAtlasSprite still = Minecraft.func_71410_x().func_147117_R().getTextureExtry(fluid.getFluid().getStill(fluid).toString());
        TextureAtlasSprite flowing = Minecraft.func_71410_x().func_147117_R().getTextureExtry(fluid.getFluid().getFlowing(fluid).toString());
        putTexturedQuad(renderer, still, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.DOWN, color, brightness, false);
        putTexturedQuad(renderer, flowing, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.NORTH, color, brightness, true);
        putTexturedQuad(renderer, flowing, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.EAST, color, brightness, true);
        putTexturedQuad(renderer, flowing, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.SOUTH, color, brightness, true);
        putTexturedQuad(renderer, flowing, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.WEST, color, brightness, true);
        putTexturedQuad(renderer, still, x1, y1, z1, x2 - x1, y2 - y1, z2 - z1, EnumFacing.UP, color, brightness, false);
        Tessellator.func_178181_a().func_78381_a();
    }

    public static void renderStackedFluidCuboid(FluidStack fluid, double px, double py, double pz, BlockPos pos, BlockPos from, BlockPos to, double ymin, double ymax) {
        if (ymin >= ymax) {
            return;
        }

        BufferBuilder renderer = Tessellator.func_178181_a().func_178180_c();
        renderer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
        Minecraft.func_71410_x().field_71446_o.func_110577_a(TextureMap.field_110575_b);
        int color = fluid.getFluid().getColor(fluid);
        int brightness = Minecraft.func_71410_x().field_71441_e.func_175626_b(pos, fluid.getFluid().getLuminosity());
        GlStateManager.func_179109_b(from.func_177958_n(), from.func_177956_o(), from.func_177952_p());
        TextureAtlasSprite still = Minecraft.func_71410_x().func_147117_R().getTextureExtry(fluid.getFluid().getStill(fluid).toString());
        TextureAtlasSprite flowing = Minecraft.func_71410_x().func_147117_R().getTextureExtry(fluid.getFluid().getFlowing(fluid).toString());

        if (still == null) {
            still = Minecraft.func_71410_x().func_147117_R().func_174944_f();
        }

        if (flowing == null) {
            flowing = Minecraft.func_71410_x().func_147117_R().func_174944_f();
        }

        int xd = to.func_177958_n() - from.func_177958_n();
        int yd = (int) (ymax - ymin);
        int zd = to.func_177952_p() - from.func_177952_p();
        double xmin = fluidOffset;
        double xmax = xd + 1d - fluidOffset;
        double zmin = fluidOffset;
        double zmax = zd + 1d - fluidOffset;
        double[] xs = new double[2 + xd];
        double[] ys = new double[2 + yd];
        double[] zs = new double[2 + zd];

        xs[0] = xmin;

        for (int i = 1; i <= xd; i++) {
            xs[i] = i;
        }

        xs[xd + 1] = xmax;

        ys[0] = ymin;

        for (int i = 1; i <= yd; i++) {
            ys[i] = i;
        }

        ys[yd + 1] = ymax;

        zs[0] = zmin;

        for (int i = 1; i <= zd; i++) {
            zs[i] = i;
        }

        zs[zd + 1] = zmax;

        for (int y = 0; y <= yd; y++) {
            for (int z = 0; z <= zd; z++) {
                for (int x = 0; x <= xd; x++) {
                    double x1 = xs[x];
                    double x2 = xs[x + 1] - x1;
                    double y1 = ys[y];
                    double y2 = ys[y + 1] - y1;
                    double z1 = zs[z];
                    double z2 = zs[z + 1] - z1;

                    if (x == 0) {
                        putTexturedQuad(renderer, flowing, x1, y1, z1, x2, y2, z2, EnumFacing.WEST, color, brightness, true);
                    }
                    if (x == xd) {
                        putTexturedQuad(renderer, flowing, x1, y1, z1, x2, y2, z2, EnumFacing.EAST, color, brightness, true);
                    }
                    if (y == 0) {
                        putTexturedQuad(renderer, still, x1, y1, z1, x2, y2, z2, EnumFacing.DOWN, color, brightness, false);
                    }
                    if (y == yd) {
                        putTexturedQuad(renderer, still, x1, y1, z1, x2, y2, z2, EnumFacing.UP, color, brightness, false);
                    }
                    if (z == 0) {
                        putTexturedQuad(renderer, flowing, x1, y1, z1, x2, y2, z2, EnumFacing.NORTH, color, brightness, true);
                    }
                    if (z == zd) {
                        putTexturedQuad(renderer, flowing, x1, y1, z1, x2, y2, z2, EnumFacing.SOUTH, color, brightness, true);
                    }
                }
            }
        }

        Tessellator.func_178181_a().func_78381_a();
    }

    public static void putTexturedQuad(BufferBuilder renderer, TextureAtlasSprite sprite, double x, double y, double z, double w, double h, double d, EnumFacing face, int color, int brightness, boolean flowing) {
        int l1 = brightness >> 0x10 & 0xFFFF;
        int l2 = brightness & 0xFFFF;
        int a = color >> 24 & 0xFF;
        int r = color >> 16 & 0xFF;
        int g = color >> 8 & 0xFF;
        int b = color & 0xFF;
        putTexturedQuad(renderer, sprite, x, y, z, w, h, d, face, r, g, b, a, l1, l2, flowing);
    }

    public static void putTexturedQuad(BufferBuilder renderer, TextureAtlasSprite sprite, double x, double y, double z, double w, double h, double d, EnumFacing face, int r, int g, int b, int a, int light1, int light2, boolean flowing) {
        if (sprite == null) {
            return;
        }

        double minU;
        double maxU;
        double minV;
        double maxV;
        double size = 16f;

        if (flowing) {
            size = 8f;
        }

        double x1 = x;
        double x2 = x + w;
        double y1 = y;
        double y2 = y + h;
        double z1 = z;
        double z2 = z + d;
        double xt1 = x1 % 1d;
        double xt2 = xt1 + w;

        while (xt2 > 1f) {
            xt2 -= 1f;
        }

        double yt1 = y1 % 1d;
        double yt2 = yt1 + h;

        while (yt2 > 1f) {
            yt2 -= 1f;
        }

        double zt1 = z1 % 1d;
        double zt2 = zt1 + d;

        while (zt2 > 1f) {
            zt2 -= 1f;
        }

        if (flowing) {
            double tmp = 1d - yt1;
            yt1 = 1d - yt2;
            yt2 = tmp;
        }

        switch (face) {
            case DOWN:
            case UP:
                minU = sprite.func_94214_a(xt1 * size);
                maxU = sprite.func_94214_a(xt2 * size);
                minV = sprite.func_94207_b(zt1 * size);
                maxV = sprite.func_94207_b(zt2 * size);
                break;
            case NORTH:
            case SOUTH:
                minU = sprite.func_94214_a(xt2 * size);
                maxU = sprite.func_94214_a(xt1 * size);
                minV = sprite.func_94207_b(yt1 * size);
                maxV = sprite.func_94207_b(yt2 * size);
                break;
            case WEST:
            case EAST:
                minU = sprite.func_94214_a(zt2 * size);
                maxU = sprite.func_94214_a(zt1 * size);
                minV = sprite.func_94207_b(yt1 * size);
                maxV = sprite.func_94207_b(yt2 * size);
                break;
            default:
                minU = sprite.func_94209_e();
                maxU = sprite.func_94212_f();
                minV = sprite.func_94206_g();
                maxV = sprite.func_94210_h();
        }

        switch (face) {
            case DOWN:
                renderer.func_181662_b(x1, y1, z1).func_181669_b(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y1, z1).func_181669_b(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y1, z2).func_181669_b(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x1, y1, z2).func_181669_b(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
                break;
            case UP:
                renderer.func_181662_b(x1, y2, z1).func_181669_b(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x1, y2, z2).func_181669_b(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y2, z2).func_181669_b(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y2, z1).func_181669_b(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
                break;
            case NORTH:
                renderer.func_181662_b(x1, y1, z1).func_181669_b(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x1, y2, z1).func_181669_b(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y2, z1).func_181669_b(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y1, z1).func_181669_b(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
                break;
            case SOUTH:
                renderer.func_181662_b(x1, y1, z2).func_181669_b(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y1, z2).func_181669_b(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y2, z2).func_181669_b(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x1, y2, z2).func_181669_b(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
                break;
            case WEST:
                renderer.func_181662_b(x1, y1, z1).func_181669_b(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x1, y1, z2).func_181669_b(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x1, y2, z2).func_181669_b(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x1, y2, z1).func_181669_b(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
                break;
            case EAST:
                renderer.func_181662_b(x2, y1, z1).func_181669_b(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y2, z1).func_181669_b(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y2, z2).func_181669_b(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
                renderer.func_181662_b(x2, y1, z2).func_181669_b(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
                break;
        }
    }

}
