package com.astro.bibliotheca.util;


import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fml.common.event.FMLInterModComms;

/**
 * The purpose of this class is to show how to use and provide an interface for Thermal Expansion's IMC Recipe manipulation.
 * <p>
 * It is not the only way to add recipes to TE, but it is BY FAR the safest. Please use it.
 *
 * @author King Lemming
 */
@SuppressWarnings({"WeakerAccess", "unused"})
public final class ThermalExpansionHelper {

    private static final String MOD_ID = "thermalexpansion";

    private static final String ENERGY = "energy";
    private static final String FLUID_NAME = "fluidName";

    private static final String INPUT = "input";
    private static final String OUTPUT = "output";
    private static final String PRIMARY_INPUT = "primaryInput";
    private static final String SECONDARY_INPUT = "secondaryInput";
    private static final String PRIMARY_OUTPUT = "primaryOutput";
    private static final String SECONDARY_OUTPUT = "secondaryOutput";
    private static final String SECONDARY_CHANCE = "secondaryChance";

    private ThermalExpansionHelper() {

    }

    /**
     * MACHINES
     */

	/* FURNACE */
    public static void addFurnaceRecipe(int energy, ItemStack input, ItemStack output) {

        if (input.func_190926_b() || output.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.func_77955_b(toSend.func_74775_l(OUTPUT));
        FMLInterModComms.sendMessage(MOD_ID, "AddFurnaceRecipe", toSend);
    }

    public static void removeFurnaceRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveFurnaceRecipe", toSend);
    }

    /* PULVERIZER */
    public static void addPulverizerRecipe(int energy, ItemStack input, ItemStack primaryOutput) {

        addPulverizerRecipe(energy, input, primaryOutput, ItemStack.field_190927_a, 0);
    }

    public static void addPulverizerRecipe(int energy, ItemStack input, ItemStack primaryOutput, ItemStack secondaryOutput) {

        addPulverizerRecipe(energy, input, primaryOutput, secondaryOutput, 100);
    }

    public static void addPulverizerRecipe(int energy, ItemStack input, ItemStack primaryOutput, ItemStack secondaryOutput, int secondaryChance) {

        if (input.func_190926_b() || primaryOutput.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(PRIMARY_OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        primaryOutput.func_77955_b(toSend.func_74775_l(PRIMARY_OUTPUT));

        if (!secondaryOutput.func_190926_b()) {
            toSend.func_74782_a(SECONDARY_OUTPUT, new NBTTagCompound());
            secondaryOutput.func_77955_b(toSend.func_74775_l(SECONDARY_OUTPUT));
            toSend.func_74768_a(SECONDARY_CHANCE, secondaryChance);
        }
        FMLInterModComms.sendMessage(MOD_ID, "AddPulverizerRecipe", toSend);
    }

    public static void removePulverizerRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemovePulverizerRecipe", toSend);
    }

    /* SAWMILL */
    public static void addSawmillRecipe(int energy, ItemStack input, ItemStack primaryOutput) {

        addSawmillRecipe(energy, input, primaryOutput, ItemStack.field_190927_a, 0);
    }

    public static void addSawmillRecipe(int energy, ItemStack input, ItemStack primaryOutput, ItemStack secondaryOutput) {

        addSawmillRecipe(energy, input, primaryOutput, secondaryOutput, 100);
    }

    public static void addSawmillRecipe(int energy, ItemStack input, ItemStack primaryOutput, ItemStack secondaryOutput, int secondaryChance) {

        if (input.func_190926_b() || primaryOutput.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(PRIMARY_OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        primaryOutput.func_77955_b(toSend.func_74775_l(PRIMARY_OUTPUT));

        if (!secondaryOutput.func_190926_b()) {
            toSend.func_74782_a(SECONDARY_OUTPUT, new NBTTagCompound());
            secondaryOutput.func_77955_b(toSend.func_74775_l(SECONDARY_OUTPUT));
            toSend.func_74768_a(SECONDARY_CHANCE, secondaryChance);
        }

        FMLInterModComms.sendMessage(MOD_ID, "AddSawmillRecipe", toSend);
    }

    public static void removeSawmillRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveSawmillRecipe", toSend);
    }

    /* SMELTER */
    public static void addSmelterRecipe(int energy, ItemStack primaryInput, ItemStack secondaryInput, ItemStack primaryOutput) {

        addSmelterRecipe(energy, primaryInput, secondaryInput, primaryOutput, ItemStack.field_190927_a, 0);
    }

    public static void addSmelterRecipe(int energy, ItemStack primaryInput, ItemStack secondaryInput, ItemStack primaryOutput, ItemStack secondaryOutput) {

        addSmelterRecipe(energy, primaryInput, secondaryInput, primaryOutput, secondaryOutput, 100);
    }

    public static void addSmelterRecipe(int energy, ItemStack primaryInput, ItemStack secondaryInput, ItemStack primaryOutput, ItemStack secondaryOutput, int secondaryChance) {

        if (primaryInput.func_190926_b() || secondaryInput.func_190926_b() || primaryOutput.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(PRIMARY_INPUT, new NBTTagCompound());
        toSend.func_74782_a(SECONDARY_INPUT, new NBTTagCompound());
        toSend.func_74782_a(PRIMARY_OUTPUT, new NBTTagCompound());

        primaryInput.func_77955_b(toSend.func_74775_l(PRIMARY_INPUT));
        secondaryInput.func_77955_b(toSend.func_74775_l(SECONDARY_INPUT));
        primaryOutput.func_77955_b(toSend.func_74775_l(PRIMARY_OUTPUT));

        if (!secondaryOutput.func_190926_b()) {
            toSend.func_74782_a(SECONDARY_OUTPUT, new NBTTagCompound());
            secondaryOutput.func_77955_b(toSend.func_74775_l(SECONDARY_OUTPUT));
            toSend.func_74768_a(SECONDARY_CHANCE, secondaryChance);
        }
        FMLInterModComms.sendMessage(MOD_ID, "AddSmelterRecipe", toSend);
    }

    public static void removeSmelterRecipe(ItemStack primaryInput, ItemStack secondaryInput) {

        if (primaryInput.func_190926_b() || secondaryInput.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(PRIMARY_INPUT, new NBTTagCompound());
        toSend.func_74782_a(SECONDARY_INPUT, new NBTTagCompound());

        primaryInput.func_77955_b(toSend.func_74775_l(PRIMARY_INPUT));
        secondaryInput.func_77955_b(toSend.func_74775_l(SECONDARY_INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveSmelterRecipe", toSend);
    }

    /* INSOLATOR */
    public static void addInsolatorRecipe(int energy, ItemStack primaryInput, ItemStack secondaryInput, ItemStack primaryOutput) {

        addInsolatorRecipe(energy, primaryInput, secondaryInput, primaryOutput, ItemStack.field_190927_a, 0);
    }

    public static void addInsolatorRecipe(int energy, ItemStack primaryInput, ItemStack secondaryInput, ItemStack primaryOutput, ItemStack secondaryOutput) {

        addInsolatorRecipe(energy, primaryInput, secondaryInput, primaryOutput, secondaryOutput, 100);
    }

    public static void addInsolatorRecipe(int energy, ItemStack primaryInput, ItemStack secondaryInput, ItemStack primaryOutput, ItemStack secondaryOutput, int secondaryChance) {

        if (primaryInput.func_190926_b() || secondaryInput.func_190926_b() || primaryOutput.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(PRIMARY_INPUT, new NBTTagCompound());
        toSend.func_74782_a(SECONDARY_INPUT, new NBTTagCompound());
        toSend.func_74782_a(PRIMARY_OUTPUT, new NBTTagCompound());

        primaryInput.func_77955_b(toSend.func_74775_l(PRIMARY_INPUT));
        secondaryInput.func_77955_b(toSend.func_74775_l(SECONDARY_INPUT));
        primaryOutput.func_77955_b(toSend.func_74775_l(PRIMARY_OUTPUT));

        if (!secondaryOutput.func_190926_b()) {
            toSend.func_74782_a(SECONDARY_OUTPUT, new NBTTagCompound());
            secondaryOutput.func_77955_b(toSend.func_74775_l(SECONDARY_OUTPUT));
            toSend.func_74768_a(SECONDARY_CHANCE, secondaryChance);
        }
        FMLInterModComms.sendMessage(MOD_ID, "AddInsolatorRecipe", toSend);
    }

    public static void removeInsolatorRecipe(ItemStack primaryInput, ItemStack secondaryInput) {

        if (primaryInput.func_190926_b() || secondaryInput.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(PRIMARY_INPUT, new NBTTagCompound());
        toSend.func_74782_a(SECONDARY_INPUT, new NBTTagCompound());

        primaryInput.func_77955_b(toSend.func_74775_l(PRIMARY_INPUT));
        secondaryInput.func_77955_b(toSend.func_74775_l(SECONDARY_INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveInsolatorRecipe", toSend);
    }

    /* COMPACTOR */
    public static void addCompactorPressRecipe(int energy, ItemStack input, ItemStack output) {

        if (input.func_190926_b() || output.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.func_77955_b(toSend.func_74775_l(OUTPUT));
        FMLInterModComms.sendMessage(MOD_ID, "AddCompactorPressRecipe", toSend);
    }

    public static void addCompactorStorageRecipe(int energy, ItemStack input, ItemStack output) {

        if (input.func_190926_b() || output.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.func_77955_b(toSend.func_74775_l(OUTPUT));
        FMLInterModComms.sendMessage(MOD_ID, "AddCompactorStorageRecipe", toSend);
    }

    public static void addCompactorMintRecipe(int energy, ItemStack input, ItemStack output) {

        if (input.func_190926_b() || output.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.func_77955_b(toSend.func_74775_l(OUTPUT));
        FMLInterModComms.sendMessage(MOD_ID, "AddCompactorMintRecipe", toSend);
    }

    public static void removeCompactorPressRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveCompactorPressRecipe", toSend);
    }

    public static void removeCompactorStorageRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveCompactorStorageRecipe", toSend);
    }

    public static void removeCompactorMintRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveCompactorMintRecipe", toSend);
    }

    /* CRUCIBLE */
    public static void addCrucibleRecipe(int energy, ItemStack input, FluidStack output) {

        if (input.func_190926_b() || output == null) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.writeToNBT(toSend.func_74775_l(OUTPUT));

        FMLInterModComms.sendMessage(MOD_ID, "AddCrucibleRecipe", toSend);
    }

    public static void removeCrucibleRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveCrucibleRecipe", toSend);
    }

    /* REFINERY */
    public static void addRefineryRecipe(int energy, FluidStack input, FluidStack output, ItemStack outputItem) {

        if (input == null || output == null) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());

        if (!outputItem.func_190926_b()) {
            toSend.func_74782_a(SECONDARY_OUTPUT, new NBTTagCompound());
            outputItem.func_77955_b(toSend.func_74775_l(SECONDARY_OUTPUT));
        }
        input.writeToNBT(toSend.func_74775_l(INPUT));
        output.writeToNBT(toSend.func_74775_l(OUTPUT));

        FMLInterModComms.sendMessage(MOD_ID, "AddRefineryRecipe", toSend);
    }

    public static void removeRefineryRecipe(FluidStack input) {

        if (input == null) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.writeToNBT(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveRefineryRecipe", toSend);
    }

    /* TRANSPOSER */
    public static void addTransposerFill(int energy, ItemStack input, ItemStack output, FluidStack fluid, boolean reversible) {

        if (input.func_190926_b() || output.func_190926_b() || fluid == null) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());
        toSend.func_74782_a("fluid", new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.func_77955_b(toSend.func_74775_l(OUTPUT));
        toSend.func_74757_a("reversible", reversible);
        fluid.writeToNBT(toSend.func_74775_l("fluid"));

        FMLInterModComms.sendMessage(MOD_ID, "AddTransposerFillRecipe", toSend);
    }

    public static void addTransposerExtract(int energy, ItemStack input, ItemStack output, FluidStack fluid, int chance, boolean reversible) {

        if (input.func_190926_b() || output.func_190926_b() || fluid == null) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());
        toSend.func_74782_a("fluid", new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.func_77955_b(toSend.func_74775_l(OUTPUT));
        toSend.func_74757_a("reversible", reversible);
        toSend.func_74768_a("chance", chance);
        fluid.writeToNBT(toSend.func_74775_l("fluid"));

        FMLInterModComms.sendMessage(MOD_ID, "AddTransposerExtractRecipe", toSend);
    }

    public static void removeTransposerFill(ItemStack input, FluidStack fluid) {

        if (input.func_190926_b() || fluid == null) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a("fluid", new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        fluid.writeToNBT(toSend.func_74775_l("fluid"));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveTransposerFillRecipe", toSend);
    }

    public static void removeTransposerExtract(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveTransposerExtractRecipe", toSend);
    }

    /* CHARGER */
    public static void addChargerRecipe(int energy, ItemStack input, ItemStack output) {

        if (input.func_190926_b() || output.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74768_a(ENERGY, energy);
        toSend.func_74782_a(INPUT, new NBTTagCompound());
        toSend.func_74782_a(OUTPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        output.func_77955_b(toSend.func_74775_l(OUTPUT));
        FMLInterModComms.sendMessage(MOD_ID, "AddChargerRecipe", toSend);
    }

    public static void removeChargerRecipe(ItemStack input) {

        if (input.func_190926_b()) {
            return;
        }
        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74782_a(INPUT, new NBTTagCompound());

        input.func_77955_b(toSend.func_74775_l(INPUT));
        FMLInterModComms.sendMessage(MOD_ID, "RemoveChargerRecipe", toSend);
    }

    /**
     * DYNAMOS
     */

	/* MAGMATIC */
    public static void addMagmaticFuel(String fluidName, int energy) {

        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74778_a(FLUID_NAME, fluidName);
        toSend.func_74768_a(ENERGY, energy);

        FMLInterModComms.sendMessage(MOD_ID, "MagmaticFuel", toSend);
    }

    /* COMPRESSION */
    public static void addCompressionFuel(String fluidName, int energy) {

        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74778_a(FLUID_NAME, fluidName);
        toSend.func_74768_a(ENERGY, energy);

        FMLInterModComms.sendMessage(MOD_ID, "CompressionFuel", toSend);
    }

    /* REACTANT */
    public static void addReactantFuel(String fluidName, int energy) {

        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74778_a(FLUID_NAME, fluidName);
        toSend.func_74768_a(ENERGY, energy);

        FMLInterModComms.sendMessage(MOD_ID, "ReactantFuel", toSend);
    }

    /* COOLANT */
    public static void addCoolant(String fluidName, int energy) {

        NBTTagCompound toSend = new NBTTagCompound();

        toSend.func_74778_a(FLUID_NAME, fluidName);
        toSend.func_74768_a(ENERGY, energy);

        FMLInterModComms.sendMessage(MOD_ID, "Coolant", toSend);
    }

}
