package com.astro.bibliotheca.worldenough.impl;

import com.astro.bibliotheca.api.worldenough.DimensionLogic;
import com.astro.bibliotheca.api.worldenough.SpawnEntry;
import com.astro.bibliotheca.api.worldenough.SpawnLogic;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import net.minecraft.block.state.IBlockState;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

public class ImplDimensionLogic implements DimensionLogic {
    private final List<SpawnEntry> logic = Lists.newArrayList();
    private final SpawnLogic parent;
    private final List<IBlockState> replaceStates = Lists.newArrayList();


    public ImplDimensionLogic(SpawnLogic parent) {
        this.parent = parent;
    }

    @Override
    public DimensionLogic addEntry(@Nonnull SpawnEntry spawnEntry) {
        logic.add(spawnEntry);
        return this;
    }

    @Override
    public DimensionLogic addBlocksToReplace(IBlockState... states) {
        Collections.addAll(replaceStates, states);
        return this;
    }

    @Override
    public Collection<SpawnEntry> getEntries() {
        return ImmutableList.copyOf(logic);
    }

    @Override
    public SpawnLogic end() {
        return parent;
    }

    @Override
    public List<? extends IBlockState> getReplaceableBlocks() {
        return ImmutableList.copyOf(replaceStates);
    }

    @Override
    public void clearEntries() {
        logic.clear();
    }

    @Override
    public void clearReplaceBlocks() {
        replaceStates.clear();
    }
}
