package com.astro.bibliotheca.worldenough.json;

import com.astro.bibliotheca.Bibliotheca;
import com.astro.bibliotheca.api.worldenough.DimensionLogic;
import com.astro.bibliotheca.api.worldenough.SpawnEntry;
import com.astro.bibliotheca.api.worldenough.SpawnLogic;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.biome.Biome;
import net.minecraftforge.fml.common.registry.ForgeRegistries;
import org.apache.commons.io.FileUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class JSONReader {
    public static final File DIR = new File(".", "config/bibliotheca/worldenough");

    public static void read() {
        JsonParser parser = new JsonParser();
        if (!DIR.exists())
            DIR.mkdirs();
        if (!DIR.isDirectory()) {
            DIR.delete();
            DIR.mkdirs();
        }

        File[] files = DIR.listFiles();
        if (files == null || files.length == 0)
            return;

        Arrays.stream(files).filter(file -> file.getName().endsWith(".json")).forEach(file -> {
            try {
                JsonElement element = parser.parse(FileUtils.readFileToString(file));
                JsonObject object = element.getAsJsonObject();
                final String id = object.get("id").getAsString();
                SpawnLogic logic = Bibliotheca.API.getWorldEnoughAPI().getSpawnLogic(id);
                if (logic == null) {
                    logic = Bibliotheca.API.getWorldEnoughAPI().createSpawnLogic();
                    Bibliotheca.API.getWorldEnoughAPI().registerSpawnLogic(id, logic);
                }
                final SpawnLogic spawnLogic = logic;
                JsonArray dimensions = object.get("dimensions").getAsJsonArray();
                dimensions.forEach(dimension -> {
                    JsonObject obj = dimension.getAsJsonObject();
                    int dimID = obj.get("id").getAsInt();
                    DimensionLogic dimensionLogic = spawnLogic.getDimension(dimID);
                    dimensionLogic.clearReplaceBlocks();
                    if (obj.has("blocksToReplace")) {
                        JsonArray replaceBlocks = obj.getAsJsonArray("blocksToReplace");
                        replaceBlocks.forEach(block -> {
                            String s = block.getAsString();
                            IBlockState state = JSONUtils.getState(s);
                            if (state != null)
                                dimensionLogic.addBlocksToReplace(state);
                        });
                    } else {
                        dimensionLogic.addBlocksToReplace(Blocks.field_150348_b.func_176223_P());
                    }

                    JsonArray entries = obj.getAsJsonArray("entries");
                    dimensionLogic.clearEntries();
                    entries.forEach(entry -> {
                        JsonObject entryObj = entry.getAsJsonObject();
                        String type = entryObj.get("type").getAsString();
                        String block = entryObj.get("block").getAsString();
                        IBlockState state = JSONUtils.getState(block);
                        if (state == null)
                            return;
                        int size = entryObj.get("size").getAsInt();
                        int weight = entryObj.get("weight").getAsInt();
                        int frequency = entryObj.get("frequency").getAsInt();
                        int minHeight = entryObj.get("minHeight").getAsInt();
                        int maxHeight = entryObj.get("maxHeight").getAsInt();
                        JsonArray biomes = entryObj.getAsJsonArray("biomes");
                        List<Biome> biomeAr = new ArrayList<>();
                        for(JsonElement biomeObj : biomes) {
                            String biome = biomeObj.getAsString();
                            Biome biome2 = ForgeRegistries.BIOMES.getValue(new ResourceLocation(biome));
                            if(biome2!=null)
                                biomeAr.add(biome2);
                        }
                        Biome[] biomeArr = biomeAr.toArray(new Biome[0]);
                        if (type.equalsIgnoreCase("blob")) {
                            SpawnEntry entry1 = SpawnEntry.createBlobEntry(state, size, weight, frequency, minHeight, maxHeight, biomeArr);
                            dimensionLogic.addEntry(entry1);
                        } else {
                            //Unsupported
                        }
                    });

                });
            } catch (Exception ignored) {

            }
        });
    }
}
