package com.astro.bibliotheca.worldenough.json;

import com.astro.bibliotheca.Bibliotheca;
import com.astro.bibliotheca.api.worldenough.DimensionLogic;
import com.astro.bibliotheca.api.worldenough.SpawnEntry;
import com.astro.bibliotheca.api.worldenough.SpawnLogic;
import com.astro.bibliotheca.api.worldenough.WorldEnoughAPI;
import com.google.common.base.Charsets;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import net.minecraft.block.state.IBlockState;
import net.minecraft.world.biome.Biome;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringEscapeUtils;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Map;

public class JSONWriter {
    public static final File DIR = new File(".", "config/bibliotheca/worldenough");

    public static void write() {
        if (!DIR.exists())
            DIR.mkdirs();
        if (!DIR.isDirectory()) {
            DIR.delete();
            DIR.mkdirs();
        }

        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        WorldEnoughAPI api = Bibliotheca.API.getWorldEnoughAPI();
        for(String id : api.getAllSpawnLogic().keySet()) {
            SpawnLogic logic = api.getSpawnLogic(id);
            File file = new File(DIR, id + ".json");

            if (file.exists()) {
                continue;
            }

            Map<Integer, DimensionLogic> dimensions = logic.getAllDimensions();

            JsonObject mainObj = new JsonObject();
            mainObj.addProperty("id", id);

            JsonArray array = new JsonArray();
            for (Map.Entry<Integer, DimensionLogic> dimension : dimensions.entrySet()) {
                JsonObject object = new JsonObject();

                if (dimension.getKey() != Integer.MAX_VALUE) {
                    object.addProperty("id", dimension.getKey());
                }
                JsonArray jsonReplace = new JsonArray();
                Collection<? extends IBlockState> replaceBlocks = dimension.getValue().getReplaceableBlocks();

                for(IBlockState state : replaceBlocks) {
                    String blockName = state.func_177230_c().getRegistryName().toString();
                    String stateString = JSONUtils.serializeState(state);
                    if (state != state.func_177230_c().func_176223_P()) {
                        blockName += "#";
                        blockName += stateString;
                    }
                    jsonReplace.add(blockName);
                }

                object.add("blocksToReplace",jsonReplace);

                JsonArray jsonEntires = new JsonArray();
                Collection<SpawnEntry> entries = dimension.getValue().getEntries();

                for (SpawnEntry spawnEntry : entries) {
                    JsonObject entry = new JsonObject();

                    String blockName = spawnEntry.getState().func_177230_c().getRegistryName().toString();
                    String state = JSONUtils.serializeState(spawnEntry.getState());
                    if (spawnEntry.getState() != spawnEntry.getState().func_177230_c().func_176223_P()) {
                        blockName += "#";
                        blockName += state;
                    }

                    entry.addProperty("type", spawnEntry.getTypeString());
                    entry.addProperty("block", blockName);

                    entry.addProperty("size", spawnEntry.getSize());
                    entry.addProperty("weight", spawnEntry.getWeight());
                    entry.addProperty("frequency", spawnEntry.getFrequency());
                    entry.addProperty("min_height", spawnEntry.getMinHeight());
                    entry.addProperty("max_height", spawnEntry.getMaxHeight());

                    JsonArray biomes = new JsonArray();
                    for (Biome biome : spawnEntry.getBiomes())
                        biomes.add(biome.getRegistryName().toString());
                    entry.add("biomes", biomes);
                    jsonEntires.add(entry);
                }
                object.add("entries", jsonEntires);
                array.add(object);
            }
            mainObj.add("dimensions", array);

            String json = gson.toJson(mainObj);

            try {
                FileUtils.writeStringToFile(file, StringEscapeUtils.unescapeJson(json), Charsets.UTF_8);
            } catch (IOException ignored) {
                ignored.printStackTrace();
            }
        }
    }

}
