package com.astro.bibliotheca.worldenough.world;

import com.astro.bibliotheca.api.worldenough.DimensionLogic;
import com.astro.bibliotheca.api.worldenough.SpawnEntry;
import com.astro.bibliotheca.api.worldenough.SpawnLogic;
import com.google.common.collect.Lists;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraft.world.chunk.IChunkProvider;
import net.minecraft.world.gen.IChunkGenerator;
import net.minecraft.world.gen.feature.WorldGenMinable;
import net.minecraftforge.fml.common.IWorldGenerator;

import java.util.Collection;
import java.util.List;
import java.util.Random;

public class WorldEnoughGenerator implements IWorldGenerator {
    private SpawnLogic logic;

    public WorldEnoughGenerator(SpawnLogic logic) {
        this.logic = logic;
    }

    @Override
    public void generate(Random rand, int chunkX, int chunkZ, World world, IChunkGenerator chunkGenerator, IChunkProvider chunkProvider) {
        DimensionLogic dimensionLogic = logic.getDimension(world.field_73011_w.getDimension());
        if (dimensionLogic.getEntries().isEmpty())
            return;
        int pX = chunkX * 16;
        int pZ = chunkZ * 16;

        Collection<SpawnEntry> entries = dimensionLogic.getEntries();
        for (SpawnEntry entry : entries) {
            if (entry.getState() == null)
                continue;
            for (int i = 0; i < entry.getFrequency(); i++) {
                if (rand.nextInt(100) < entry.getWeight()) {
                    int firstBlockXCoord = pX + rand.nextInt(16);
                    int firstBlockZCoord = pZ + rand.nextInt(16);
                    int quisqueY = entry.getMinHeight() + rand.nextInt(entry.getMaxHeight() - entry.getMinHeight());

                    BlockPos quisquePos = new BlockPos(firstBlockXCoord, quisqueY, firstBlockZCoord);
                    List<Biome> biomes = Lists.newArrayList(entry.getBiomes());
                    if (biomes.isEmpty() || biomes.contains(world.func_180494_b(quisquePos))) new WorldGenMinable(entry.getState(), entry.getSize()).func_180709_b(world, rand, quisquePos);
                }
            }
        }
    }
}
